@@ -12,7 +12,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-# Copyright 2004 Duke University 
+# Copyright 2005 Duke University 
 # Written by Seth Vidal
 
 
@@ -28,6 +28,7 @@ import re
 
 import output
 from urlgrabber.progress import TextMeter
+import shell
 import yum
 import yum.Errors
 import yum.misc
@@ -42,8 +43,12 @@ import callback
 import urlgrabber
 import urlgrabber.grabber
 
-__version__ = '2.2.2'
+__version__ = '2.3.1'
 
+class CliError(yum.Errors.YumBaseError):
+   def __init__(self, args=None):
+        yum.Errors.YumBaseError.__init__(self)
+        self.args = args
 
 class YumBaseCli(yum.YumBase, output.YumOutput):
     """This is the base class for yum cli.
@@ -57,11 +62,11 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
     def doRepoSetup(self, nosack=None):
         """grabs the repomd.xml for each enabled repository and sets up the basics
            of the repository"""
-
+        
         if hasattr(self, 'pkgSack'):
             self.log(7, 'skipping reposetup, pkgsack exists')
             return
-           
+            
         self.log(2, 'Setting up Repos')
         if len(self.repos.listEnabled()) < 1:
             self.errorlog(0, 'No Repositories Available to Set Up')
@@ -115,7 +120,8 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         except getopt.error, e:
             self.errorlog(0, _('Options Error: %s') % e)
             self.usage()
-    
+            sys.exit(1)
+            
         # get the early options out of the way
         # these are ones that:
         #  - we need to know about and do NOW
@@ -147,7 +153,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                 root = installroot
                     
             try:
-                self.conf = yumconf(configfile = conffile, root = root)
+                self.doConfigSetup(fn = conffile, root = root)
             except yum.Errors.ConfigError, e:
                 self.errorlog(0, _('Config Error: %s') % e)
                 sys.exit(1)
@@ -203,6 +209,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                     self.conf.setConfigOption('assumeyes',1)
                 elif o in ['-h', '--help']:
                     self.usage()
+                    sys.exit(0)
                 elif o == '-C':
                     self.conf.setConfigOption('cache', 1)
                 elif o == '-R':
@@ -215,16 +222,18 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                     self.conf.setConfigOption('rss-filename', a)
                 elif o == '--enablerepo':
                     try:
-                        self.conf.repos.enableRepo(a)
+                        self.repos.enableRepo(a)
                     except yum.Errors.ConfigError, e:
                         self.errorlog(0, _(e))
                         self.usage()
+                        sys.exit(1)
                 elif o == '--disablerepo':
                     try:
-                        self.conf.repos.disableRepo(a)
+                        self.repos.disableRepo(a)
                     except yum.Errors.ConfigError, e:
                         self.errorlog(0, _(e))
                         self.usage()
+                        sys.exit(1)
                         
                 elif o == '--exclude':
                     try:
@@ -234,33 +243,33 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                     except yum.Errors.ConfigError, e:
                         self.errorlog(0, _(e))
                         self.usage()
+                        sys.exit(1)
                 
                             
         except ValueError, e:
             self.errorlog(0, _('Options Error: %s') % e)
             self.usage()
-        
+            sys.exit(1)
+            
         # if we're below 2 on the debug level we don't need to be outputting
         # progress bars - this is hacky - I'm open to other options
         # One of these is a download
         if self.conf.getConfigOption('debuglevel') < 2 or not sys.stdout.isatty():
-            self.conf.repos.setProgressBar(None)
-            self.conf.repos.callback = None
+            self.repos.setProgressBar(None)
+            self.repos.callback = None
         else:
-            self.conf.repos.setProgressBar(TextMeter(fo=sys.stdout))
-            self.conf.repos.callback = self.simpleProgressBar
+            self.repos.setProgressBar(TextMeter(fo=sys.stdout))
+            self.repos.callback = output.CacheProgressCallback(self.log,
+                    self.errorlog, self.filelog)
 
         # setup our failure report for failover
         freport = (self.failureReport,(),{})
-        self.conf.repos.setFailureCallback(freport)
+        self.repos.setFailureCallback(freport)
         
         # setup our depsolve progress callback
         dscb = output.DepSolveProgressCallBack(self.log, self.errorlog)
         self.dsCallback = dscb
         
-        # this is just a convenience reference
-        self.repos = self.conf.repos
-        
         # save our original args out
         self.args = args
         # save out as a nice command string
@@ -268,8 +277,11 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         for arg in self.args:
             self.cmdstring += '%s ' % arg
 
-        self.parseCommands() # before we exit check over the base command + args
-                             # make sure they match
+        try:
+            self.parseCommands() # before we exit check over the base command + args
+                             # make sure they match/make sense
+        except CliError:
+            sys.exit(1)
     
         # set our caching mode correctly
         
@@ -279,7 +291,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         time.sleep(sleeptime)
 
 
-    def parseCommands(self):
+    def parseCommands(self, mycommands=[]):
         """reads self.cmds and parses them out to make sure that the requested 
         base command + argument makes any sense at all""" 
 
@@ -293,7 +305,8 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         if len(self.cmds) < 1:
             self.errorlog(0, _('You need to give some command'))
             self.usage()
-
+            raise CliError
+            
         self.basecmd = self.cmds[0] # our base command
         self.extcmds = self.cmds[1:] # out extended arguments/commands
         
@@ -308,9 +321,9 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                                 'clean', 'remove', 'provides', 'check-update',
                                 'search', 'generate-rss', 'upgrade', 
                                 'whatprovides', 'localinstall', 'localupdate',
-                                'resolvedep']:
+                                'resolvedep', 'shell']:
             self.usage()
-            
+            raise CliError
     
         if self.conf.getConfigOption('uid') != 0:
             if self.basecmd in ['install', 'update', 'clean', 'upgrade','erase', 
@@ -318,7 +331,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                                 'groupremove', 'importkey', 'makecache', 
                                 'localinstall', 'localupdate']:
                 self.errorlog(0, _('You need to be root to perform this command.'))
-                sys.exit(1)
+                raise CliError
 
         if self.basecmd in ['install', 'update', 'upgrade', 'groupinstall',
                             'groupupdate', 'localinstall', 'localupdate']:
@@ -341,31 +354,36 @@ will install it for you.
 For more information contact your distribution or package provider.
 """)
                         self.errorlog(0, msg)
-                        sys.exit(1)
+                        raise CliError
 
                 
         if self.basecmd in ['install', 'erase', 'remove', 'localinstall', 'localupdate']:
             if len(self.extcmds) == 0:
                 self.errorlog(0, _('Error: Need to pass a list of pkgs to %s') % self.basecmd)
                 self.usage()
+                raise CliError
     
         elif self.basecmd in ['provides', 'search', 'whatprovides']:
             if len(self.extcmds) == 0:
                 self.errorlog(0, _('Error: Need an item to match'))
                 self.usage()
-            
+                raise CliError
+                
         elif self.basecmd in ['groupupdate', 'groupinstall', 'groupremove', 'groupinfo']:
             if len(self.extcmds) == 0:
                 self.errorlog(0, _('Error: Need a group or list of groups'))
                 self.usage()
-    
+                raise CliError
+                
         elif self.basecmd == 'clean':
             if len(self.extcmds) == 0:
                 self.errorlog(0,
                     _('Error: clean requires an option: headers, packages, cache, metadata, all'))
             for cmd in self.extcmds:
-                if cmd not in ['headers', 'packages', 'metadata', 'cache', 'all']:
+                if cmd not in ['headers', 'packages', 'metadata', 'cache', 'dbcache', 'all']:
                     self.usage()
+                    raise CliError
+                    
         elif self.basecmd == 'generate-rss':
             if len(self.extcmds) == 0:
                 self.extcmds.insert(0, 'recent')
@@ -373,7 +391,24 @@ For more information contact your distribution or package provider.
             if self.extcmds[0] not in ['updates', 'recent']:
                 self.errorlog(0, _("Error: generate-rss takes no argument, 'updates' or 'recent'."))
                 self.usage()
+                raise CliError
             
+        elif self.basecmd == 'shell':
+            if len(self.extcmds) == 0:
+                self.log(3, "No argument to shell")
+                pass
+            elif len(self.extcmds) == 1:
+                self.log(3, "Filename passed to shell: %s" % self.extcmds[0])              
+                if not os.path.isfile(self.extcmds[0]):
+                    self.errorlog(
+                        0, _("File: %s given has argument to shell does not exists." % self.extcmds))
+                    self.usage()
+                    raise CliError
+            else:
+                self.errorlog(0,_("Error: more than one file given as argument to shell."))
+                self.usage()
+                raise CliError
+              
         elif self.basecmd in ['list', 'check-update', 'info', 'update', 'upgrade',
                               'generate-rss', 'grouplist', 'makecache',
                               'resolvedep']:
@@ -381,6 +416,21 @@ For more information contact your distribution or package provider.
     
         else:
             self.usage()
+            raise CliError
+
+    def doShell(self):
+        """do a shell-like interface for yum commands"""
+
+        self.log(2, 'Setting up Yum Shell')
+        self.doTsSetup()
+        self.doRpmDBSetup()
+        if len(self.extcmds) == 0:
+            yumshell = shell.YumShell(base=self)
+            yumshell.cmdloop()
+        else:
+            yumshell = shell.YumShell(base=self)
+            yumshell.script()
+        return yumshell.result, yumshell.resultmsgs
 
     def doCommands(self):
         """calls the base command passes the extended commands/args out to be
@@ -398,8 +448,7 @@ For more information contact your distribution or package provider.
             self.doTsSetup()
         except yum.Errors.YumBaseError, e:
             return 1, [str(e)]
-
-
+        
         if self.basecmd == 'install':
             self.log(2, "Setting up Install Process")
             try:
@@ -506,36 +555,7 @@ For more information contact your distribution or package provider.
                 
         elif self.basecmd == 'clean':
             self.conf.setConfigOption('cache', 1)
-            hdrcode = pkgcode = xmlcode = piklcode = 0
-            pkgresults = hdrresults = xmlresults = piklresults = []
-
-            if 'all' in self.extcmds:
-                self.log(2, 'Cleaning up Everything')
-                pkgcode, pkgresults = self.cleanPackages()
-                hdrcode, hdrresults = self.cleanHeaders()
-                xmlcode, xmlresults = self.cleanMetadata()
-                piklcode, piklresults = self.cleanPickles()
-                
-                code = hdrcode + pkgcode + xmlcode + piklcode
-                results = hdrresults + pkgresults + xmlresults + piklresults
-                return code, results
-            if 'headers' in self.extcmds:
-                self.log(2, 'Cleaning up Headers')
-                hdrcode, hdrresults = self.cleanHeaders()
-            if 'packages' in self.extcmds:
-                self.log(2, 'Cleaning up Packages')
-                pkgcode, pkgresults = self.cleanPackages()
-            if 'metadata' in self.extcmds:
-                self.log(2, 'Cleaning up xml metadata')
-                xmlcode, xmlresults = self.cleanMetadata()
-            if 'cache' in self.extcmds:
-                self.log(2, 'Cleaning up pickled cache')
-                piklcode, piklresults =  self.cleanPickles()
-                
-            code = hdrcode + pkgcode + xmlcode + piklcode
-            results = hdrresults + pkgresults + xmlresults + piklresults
-            return code, results
-            
+            return self.cleanCli()
         
         elif self.basecmd in ['groupupdate', 'groupinstall', 'groupremove', 
                               'grouplist', 'groupinfo']:
@@ -625,7 +645,7 @@ For more information contact your distribution or package provider.
         downloadpkgs = []
         for txmbr in self.tsInfo.getMembers():
             if txmbr.ts_state in ['i', 'u']:
-                po = self.getPackageObject(txmbr.pkgtup)
+                po = txmbr.po
                 if po:
                     downloadpkgs.append(po)
 
@@ -668,10 +688,15 @@ For more information contact your distribution or package provider.
         del self.ts
         
         self.initActionTs()
+        # save our dsCallback out
+        dscb = self.dsCallback
         self.dsCallback = None # dumb, dumb dumb dumb!
         self.populateTs(keepold=0) # sigh
         tserrors = self.ts.test(testcb, conf=tsConf)
         del testcb
+        # put it back
+        self.dsCallback = dscb
+        
         self.log(2, 'Finished Transaction Test')
         if len(tserrors) > 0:
             errstring = 'Transaction Check Error: '
@@ -829,7 +854,6 @@ For more information contact your distribution or package provider.
             if len(unmatched) > 0: # if we get back anything in unmatched, check it for a virtual-provide
                 arg = unmatched[0] #only one in there
                 self.log(3, 'Checking for virtual provide or file-provide for %s' % arg)
-            
                 try:
                     mypkg = self.returnPackageByDep(arg)
                 except yum.Errors.YumBaseError, e:
@@ -843,12 +867,12 @@ For more information contact your distribution or package provider.
                     matched.extend(mtch)
             
             installable = yum.misc.unique(exactmatch + matched)
-            exactarch = self.conf.getConfigOption('exactarch')
+            exactarch = self.conf.exactarch
             
             # we look through each returned possibility and rule out the
             # ones that we obviously can't use
             for pkg in installable:
-                if pkg.pkgtup() in installed:
+                if pkg.pkgtup in installed:
                     self.log(6, 'Package %s is already installed, skipping' % pkg)
                     continue
                 
@@ -871,9 +895,9 @@ For more information contact your distribution or package provider.
                         if rc < 0: # we're newer - this is an update, pass to them
                             if exactarch:
                                 if pkg.arch == a2:
-                                    passToUpdate.append(pkg.pkgtup())
+                                    passToUpdate.append(pkg.pkgtup)
                             else:
-                                passToUpdate.append(pkg.pkgtup())
+                                passToUpdate.append(pkg.pkgtup)
                         elif rc == 0: # same, ignore
                             continue
                         elif rc > 0: # lesser, check if the pkgtup is an exactmatch
@@ -881,13 +905,13 @@ For more information contact your distribution or package provider.
                                         # the user explicitly wants this version
                                         # FIXME this is untrue if the exactmatch
                                         # does not include a version-rel section
-                            if pkg.pkgtup() in exactmatch:
+                            if pkg.pkgtup in exactmatch:
                                 if not toBeInstalled.has_key(pkg.name): toBeInstalled[pkg.name] = []
-                                toBeInstalled[pkg.name].append(pkg.pkgtup())
+                                toBeInstalled[pkg.name].append(pkg.pkgtup)
                 else: # we've not got any installed that match n or n+a
                     self.log(4, 'No other %s installed, adding to list for potential install' % pkg.name)
                     if not toBeInstalled.has_key(pkg.name): toBeInstalled[pkg.name] = []
-                    toBeInstalled[pkg.name].append(pkg.pkgtup())
+                    toBeInstalled[pkg.name].append(pkg.pkgtup)
         
         
         # this is where I could catch the installs of compat and multilib 
@@ -987,7 +1011,7 @@ For more information contact your distribution or package provider.
             updateMatches = yum.misc.unique(matched + exactmatch)
             for po in updateMatches:
                 for (new, old) in updates:
-                    if po.pkgtup() == new:
+                    if po.pkgtup == new:
                         updated_pkg = YumInstalledPackage(self.rpmdb.returnHeaderByTuple(old)[0])
                         self.tsInfo.addUpdate(po, updated_pkg)
 
@@ -1073,8 +1097,8 @@ For more information contact your distribution or package provider.
 
             for instTup in installedByKey:
                 installed_pkg = YumInstalledPackage(self.rpmdb.returnHeaderByTuple(instTup)[0])
-                (n, a, e, v, r) = po.pkgtup()
-                (n2, a2, e2, v2, r2) = installed_pkg.pkgtup()
+                (n, a, e, v, r) = po.pkgtup
+                (n2, a2, e2, v2, r2) = installed_pkg.pkgtup
                 rc = compareEVR((e2, v2, r2), (e, v, r))
                 if rc < 0: # we're newer - this is an update, pass to them
                     if self.conf.exactarch:
@@ -1211,7 +1235,49 @@ For more information contact your distribution or package provider.
                 self.log(0, msg)
 
         return 0, []
+    
+    def cleanCli(self, userlist=None):
+        if userlist is None:
+            userlist = self.extcmds
+        hdrcode = pkgcode = xmlcode = piklcode = dbcode = 0
+        pkgresults = hdrresults = xmlresults = piklresults = dbresults = []
+
+        if 'all' in self.extcmds:
+            self.log(2, 'Cleaning up Everything')
+            pkgcode, pkgresults = self.cleanPackages()
+            hdrcode, hdrresults = self.cleanHeaders()
+            xmlcode, xmlresults = self.cleanMetadata()
+            dbcode, dbresults = self.cleanSqlite()
+            piklcode, piklresults = self.cleanPickles()
             
+            code = hdrcode + pkgcode + xmlcode + piklcode + dbcode
+            results = hdrresults + pkgresults + xmlresults + piklresults + dbresults
+            for msg in results:
+                self.log(2, msg)
+            return code, []
+            
+        if 'headers' in self.extcmds:
+            self.log(2, 'Cleaning up Headers')
+            hdrcode, hdrresults = self.cleanHeaders()
+        if 'packages' in self.extcmds:
+            self.log(2, 'Cleaning up Packages')
+            pkgcode, pkgresults = self.cleanPackages()
+        if 'metadata' in self.extcmds:
+            self.log(2, 'Cleaning up xml metadata')
+            xmlcode, xmlresults = self.cleanMetadata()
+        if 'cache' in self.extcmds:
+            self.log(2, 'Cleaning up pickled cache')
+            piklcode, piklresults =  self.cleanPickles()
+        if 'dbcache' in self.extcmds:
+            self.log(2, 'Cleaning up database cache')
+            dbcode, dbresults =  self.cleanSqlite()
+            
+        code = hdrcode + pkgcode + xmlcode + piklcode + dbcode
+        results = hdrresults + pkgresults + xmlresults + piklresults + dbresults
+        for msg in results:
+            self.log(2, msg)
+        return code, []
+
     def returnGroupLists(self, userlist=None):
 
         uservisible=1
@@ -1262,7 +1328,7 @@ For more information contact your distribution or package provider.
         installed = self.rpmdb.getPkgList()
         availablepackages = {}
         for po in self.pkgSack.returnPackages():
-            if po.pkgtup() not in installed:
+            if po.pkgtup not in installed:
                     availablepackages[po.name] = 1
 
         for group in grouplist:
@@ -1381,7 +1447,7 @@ For more information contact your distribution or package provider.
     Usage:  yum [options] < update | install | info | remove | list |
             clean | provides | search | check-update | groupinstall | 
             groupupdate | grouplist | groupinfo | groupremove | generate-rss |
-            makecache | localinstall >
+            makecache | localinstall | shell >
                 
         Options:
         -c [config file] - specify the config file to use
@@ -1399,7 +1465,7 @@ For more information contact your distribution or package provider.
 
         -h, --help  - this screen
     """)
-        sys.exit(1)
+
 
            
 
