@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-# Copyright 2004 Duke University 
+# Copyright 2005 Duke University 
 
 import os
 import os.path
@@ -46,35 +46,7 @@ class YumOutput:
     
         
     def simpleProgressBar(self, current, total, name=None):
-        """simple progress bar 50 # marks"""
-        
-        mark = '#'
-        if not sys.stdout.isatty():
-            return
-            
-        if current == 0:
-            percent = 0 
-        else:
-            if total != 0:
-                percent = current*100/total
-            else:
-                percent = 0
-    
-        numblocks = int(percent/2)
-        hashbar = mark * numblocks
-        if name is None:
-            output = '\r%-50s %d/%d' % (hashbar, current, total)
-        else:
-            output = '\r%-10.10s: %-50s %d/%d' % (name, hashbar, current, total)
-         
-        if current <= total:
-            sys.stdout.write(output)
-    
-        if current == total:
-            sys.stdout.write('\n')
-    
-        sys.stdout.flush()
-        
+        progressbar(current, total, name)
     
     def simpleList(self, pkg):
         ver = pkg.printVer()
@@ -208,7 +180,7 @@ class YumOutput:
         # build up changelog
         changelog = ''
         cnt = 0
-        for e in pkg.changelog:
+        for e in pkg.returnChangelog():
             cnt += 1
             if cnt > 3: 
                 changelog += '...'
@@ -389,7 +361,7 @@ class YumOutput:
             (n,a,e,v,r) = txmbr.pkgtup
             obspkg = None
             for (pkg, relationship) in txmbr.relatedto:
-                if relationship == 'obsoletedby':
+                if relationship == 'osboletedby':
                     obspkg = '%s.%s %s:%s-%s' % pkg
             if obspkg is not None:
                 otherout = otherout + "  Obsoleting: %s.%s %s:%s-%s with %s\n" % (n, a, e, v, r, obspkg)
@@ -509,4 +481,61 @@ class DepSolveProgressCallBack:
     
     def downloadHeader(self, name):
         self.log(2, '---> Downloading header for %s to pack into transaction set.' % name)
+       
+
+class CacheProgressCallback:
+
+    '''
+    The class handles text output callbacks during metadata cache updates.
+    '''
+    
+    def __init__(self, log, errorlog, filelog=None):
+        self.log = log
+        self.errorlog = errorlog
+        self.filelog = filelog
+
+    def log(self, level, message):
+        self.log(level, message)
+
+    def errorlog(self, level, message):
+        if self.errorlog:
+            self.errorlog(level, message)
+
+    def filelog(self, level, message):
+        if self.filelog:
+            self.filelog(level, message)
+
+    def progressbar(self, current, total, name=None):
+        progressbar(current, total, name)
+
+
+def progressbar(current, total, name=None):
+    """simple progress bar 50 # marks"""
+    
+    mark = '#'
+    if not sys.stdout.isatty():
+        return
+        
+    if current == 0:
+        percent = 0 
+    else:
+        if total != 0:
+            percent = current*100/total
+        else:
+            percent = 0
+
+    numblocks = int(percent/2)
+    hashbar = mark * numblocks
+    if name is None:
+        output = '\r%-50s %d/%d' % (hashbar, current, total)
+    else:
+        output = '\r%-10.10s: %-50s %d/%d' % (name, hashbar, current, total)
+     
+    if current <= total:
+        sys.stdout.write(output)
+
+    if current == total:
+        sys.stdout.write('\n')
+
+    sys.stdout.flush()
         
