@@ -25,6 +25,7 @@ import types
 import errno
 import time
 import sre_constants
+import glob
 
 import Errors
 import rpmUtils
@@ -32,6 +33,7 @@ import rpmUtils.updates
 import rpmUtils.arch
 import groups
 import config
+import repos
 import transactioninfo
 from urlgrabber.grabber import URLGrabError
 import depsolve
@@ -48,6 +50,9 @@ class YumBase(depsolve.Depsolve):
     def __init__(self):
         depsolve.Depsolve.__init__(self)
         self.localdbimported = 0
+        self.repos = repos.RepoStorage() # class of repositories
+        if (not self.repos.sqlite):
+            self.log(1,"Warning, could not load sqlite, falling back to pickle")
     
     def log(self, value, msg):
         """dummy log stub"""
@@ -60,12 +65,65 @@ class YumBase(depsolve.Depsolve):
     def filelog(self, value, msg):
         print msg
     
-    def doConfigSetup(self, fn='/etc/yum.conf'):
+    def doConfigSetup(self, fn='/etc/yum.conf', root='/'):
         """basic stub function for doing configuration setup"""
         
-        self.conf = config.yumconf(configfile=fn)
-        self.repos = self.conf.repos
+        self.conf = config.yumconf(configfile=fn, root=root)
+        self.getReposFromConfig()
+
+    def getReposFromConfig(self):
+        """read in repositories from config main and .repo files"""
         
+        reposlist = []
+        # look through our repositories.
+        for section in self.conf.cfg.sections(): # loop through the list of sections
+            if section != 'main': # must be a repoid
+                try:
+                    thisrepo = config.cfgParserRepo(section, self.conf, self.conf.cfg)
+                except (Errors.RepoError, Errors.ConfigError), e:
+                    self.errorlog(2, e)
+                    continue
+                else:
+                    reposlist.append(thisrepo)
+
+        # reads through reposdir for *.repo
+        # does not read recursively
+        # read each of them in using confpp, then parse them same as any other repo
+        # section - as above.
+        reposdir = self.conf.reposdir
+        if os.path.exists(self.conf.installroot + '/' + reposdir):
+            reposdir = self.conf.installroot + '/' + reposdir
+        
+        reposglob = reposdir + '/*.repo'
+        if os.path.exists(reposdir) and os.path.isdir(reposdir):
+            repofn = glob.glob(reposglob)
+            repofn.sort()
+            
+            for fn in repofn:
+                if not os.path.isfile(fn):
+                    continue
+                try:
+                    cfg, sections = config.parseDotRepo(fn)
+                except Errors.ConfigError, e:
+                    self.errorlog(2, e)
+                    continue
+
+                for section in sections:
+                    try:
+                        thisrepo = config.cfgParserRepo(section, self.conf, cfg)
+                    except (Errors.RepoError, Errors.ConfigError), e:
+                        self.errorlog(2, e)
+                        continue
+                    else:
+                        reposlist.append(thisrepo)
+
+        # got our list of repo objects
+        for thisrepo in reposlist:
+            try:
+                self.repos.add(thisrepo)
+            except Errors.RepoError, e: 
+                self.errorlog(2, e)
+                continue
         
     def doTsSetup(self):
         """setup all the transaction set storage items we'll need
@@ -96,13 +154,15 @@ class YumBase(depsolve.Depsolve):
         """closes down the instances of the rpmdb we have wangling around"""
         if hasattr(self, 'rpmdb'):
             del self.rpmdb
-            
+            self.localdbimported = 0
         if hasattr(self, 'ts'):
             del self.ts.ts
             del self.ts
         if hasattr(self, 'read_ts'):
             del self.read_ts.ts
             del self.read_ts
+        if hasattr(self, 'up'):
+            del self.up
 
     def doRepoSetup(self, nosack=None):
         """grabs the repomd.xml for each enabled repository and sets up the basics
@@ -143,6 +203,7 @@ class YumBase(depsolve.Depsolve):
         self.repos.populateSack()
         self.pkgSack = self.repos.pkgSack
         self.excludePackages()
+        self.excludeNonCompatArchs(archlist=archlist)
         for repo in self.repos.listEnabled():
             self.excludePackages(repo)
             self.includePackages(repo)
@@ -215,12 +276,6 @@ class YumBase(depsolve.Depsolve):
     def buildTransaction(self):
         """go through the packages in the transaction set, find them in the
            packageSack or rpmdb, and pack up the ts accordingly"""
-        #FIXME - dup this function out into cli.py to add callbacks for
-        # the depresolution process
-        # callbacks should be:
-        # - each package added to ts
-        # - each dep processed
-        # - each restart of dep resolution loop
         (rescode, restring) = self.resolveDeps()
         return rescode, restring
 
@@ -285,19 +340,15 @@ class YumBase(depsolve.Depsolve):
             
         self.log(2, 'Finished')
         
-    def excludeNonCompatArchs(self):
+    def excludeNonCompatArchs(self, archlist=None):
         """runs through the whole packageSack and excludes any arch not compatible
            with the system"""
         
-        self.log(2, 'Excluding Incompatible Archs')
-        archlist = ['src'] # source rpms are allowed
-        archlist.extend(rpmUtils.arch.getArchList())
-        
-        for po in self.pkgSack.returnPackages():
-            if po.arch not in archlist:
-                self.log(3, 'Arch Excluding %s' % po)
-                self.pkgSack.delPackage(po)
-        self.log(2, 'Finished')
+        self.log(3, 'Excluding Incompatible Archs')
+        if not archlist:
+            archlist.extend(rpmUtils.arch.getArchList())
+        self.pkgSack.excludeArchs(archlist)
+        self.log(3, 'Finished')
         
         
         
@@ -307,10 +358,7 @@ class YumBase(depsolve.Depsolve):
         # if we're not root then we don't lock - just return nicely
         if self.conf.getConfigOption('uid') != 0:
             return
-
-        root = self.conf.installroot
-        lockfile = root + '/' + lockfile # lock in the chroot
-
+        
         mypid=str(os.getpid())    
         while not self._lock(lockfile, mypid, 0644):
             fd = open(lockfile, 'r')
@@ -339,17 +387,10 @@ class YumBase(depsolve.Depsolve):
         # if we're not root then we don't lock - just return nicely
         if self.conf.getConfigOption('uid') != 0:
             return
-
-        root = self.conf.installroot
-        lockfile = root + '/' + lockfile # lock in the chroot
-
         self._unlock(lockfile)
         
     def _lock(self, filename, contents='', mode=0777):
-        lockdir = os.path.dirname(filename)
         try:
-            if not os.path.exists(lockdir):
-                os.makedirs(lockdir, mode=0755)
             fd = os.open(filename, os.O_EXCL|os.O_CREAT|os.O_WRONLY, mode)
         except OSError, msg:
             if not msg.errno == errno.EEXIST: raise msg
@@ -389,7 +430,7 @@ class YumBase(depsolve.Depsolve):
                 return 0
 
         ylp = YumLocalPackage(self.read_ts, fo)
-        if ylp.pkgtup() != po.pkgtup():
+        if ylp.pkgtup != po.pkgtup:
             if raiseError:
                 raise URLGrabError(-1, 'Package does not match intended download')
             else:
@@ -482,7 +523,7 @@ class YumBase(depsolve.Depsolve):
                 
         yip = YumInstalledPackage(hdr) # we're using YumInstalledPackage b/c
                                        # it takes headers <shrug>
-        if yip.pkgtup() != po.pkgtup():
+        if yip.pkgtup != po.pkgtup:
             if raiseError:
                 raise URLGrabError(-1, 'Header does not match intended download')
             else:
@@ -657,6 +698,26 @@ class YumBase(depsolve.Depsolve):
         msg = '%d cache files removed' % removed
         return 0, [msg]
     
+    def cleanSqlite(self):
+        filelist = []
+        ext = 'sqlite'
+        removed = 0
+        for repo in self.repos.listEnabled():
+            repo.dirSetup()
+            path = repo.cachedir
+            filelist = misc.getFileList(path, ext, filelist)
+            
+        for item in filelist:
+            try:
+                os.unlink(item)
+            except OSError, e:
+                self.errorlog(0, 'Cannot remove sqlite cache file %s' % item)
+                continue
+            else:
+                self.log(7, 'Cache file %s removed' % item)
+                removed+=1
+        msg = '%d cache files removed' % removed
+        return 0, [msg]
 
     def cleanMetadata(self):
         filelist = []
@@ -680,7 +741,7 @@ class YumBase(depsolve.Depsolve):
                 removed+=1
         msg = '%d metadata files removed' % removed
         return 0, [msg]
-        
+
     def sortPkgObj(self, pkg1 ,pkg2):
         """sorts a list of package tuples by name"""
         if pkg1.name > pkg2.name:
@@ -689,7 +750,7 @@ class YumBase(depsolve.Depsolve):
             return 0
         else:
             return -1
-    
+        
     def doPackageLists(self, pkgnarrow='all'):
         """generates lists of packages, un-reduced, based on pkgnarrow option"""
         
@@ -769,7 +830,7 @@ class YumBase(depsolve.Depsolve):
             avail = self.pkgSack.simplePkgList()
             for hdr in self.rpmdb.getHdrList():
                 po = YumInstalledPackage(hdr)
-                if po.pkgtup() not in avail:
+                if po.pkgtup not in avail:
                     extras.append(po)
 
         # obsoleting packages (and what they obsolete)
@@ -914,7 +975,16 @@ class YumBase(depsolve.Depsolve):
                 raise Errors.MiscError, \
                  'Search Expression: %s is an invalid Regular Expression.\n' % arg
             
-            for po in self.pkgSack:
+            # If this is not a regular expression, only search in packages
+            # returned by pkgSack.searchAll
+            if restring.find('*') == restring.find('?') \
+              == restring.find('%') == -1 and \
+              hasattr(self.pkgSack,'searchAll'):
+                where = self.pkgSack.searchAll(restring)
+            else:
+                where = self.pkgSack
+
+            for po in where:
                 tmpvalues = []
                 for filetype in po.returnFileTypes():
                     for fn in po.returnFileEntries(ftype=filetype):
@@ -1008,7 +1078,7 @@ class YumBase(depsolve.Depsolve):
         
         # look it up in the self.localPackages first:
         for po in self.localPackages:
-            if po.pkgtup() == pkgtup:
+            if po.pkgtup == pkgtup:
                 return po
                 
         pkgs = self.pkgSack.searchNevra(name=n, arch=a, epoch=e, ver=v, rel=r)
@@ -1055,7 +1125,6 @@ class YumBase(depsolve.Depsolve):
            pass back the best(or first) package it finds providing that dep."""
         
         flags = {'>':'GT', '<':'LT', '=': 'EQ', '==': 'EQ', '>=':'GE', '<=':'LE'}
-
         self.doRepoSetup()
         # parse the string out
         #  either it is 'dep (some operator) e:v-r'
