@@ -21,8 +21,6 @@ import os.path
 import types
 import re
 import urlparse
-import fnmatch
-
 
 import Errors
 from urlgrabber.grabber import URLGrabber
@@ -34,9 +32,6 @@ from repomd import packageSack
 from packages import YumAvailablePackage
 import mdcache
 
-_is_fnmatch_pattern = re.compile(r"[*?[]").search
-
-
 class YumPackageSack(packageSack.PackageSack):
     """imports/handles package objects from an mdcache dict object"""
     def __init__(self, packageClass):
@@ -54,7 +49,7 @@ class YumPackageSack(packageSack.PackageSack):
             current = 0        
             for pkgid in dataobj.keys():
                 current += 1
-                if callback: callback(current, total, repoid)
+                if callback: callback.progressbar(current, total, repoid)
                 pkgdict = dataobj[pkgid]
                 po = self.pc(pkgdict, repoid)
                 po.simple['id'] = pkgid
@@ -75,7 +70,7 @@ class YumPackageSack(packageSack.PackageSack):
             current = 0
             for pkgid in dataobj.keys():
                 current += 1
-                if callback: callback(current, total, repoid)
+                if callback: callback.progressbar(current, total, repoid)
                 pkgdict = dataobj[pkgid]
                 if self.pkgsByID.has_key(pkgid):
                     for po in self.pkgsByID[pkgid]:
@@ -95,21 +90,33 @@ class RepoStorage:
     def __init__(self):
         self.repos = {} # list of repos by repoid pointing a repo object 
                         # of repo options/misc data
-        self.pkgSack = YumPackageSack(YumAvailablePackage)
         self.callback = None # progress callback used for populateSack() for importing the xml files
         self.cache = 0
+        # Check to see if we can import sqlite stuff
+        try:
+            import sqlitecache
+            import sqlitesack
+        except ImportError:
+            self.sqlite = False
+        else:
+            self.sqlite = True
+            self.sqlitecache = sqlitecache
+            
+        if (self.sqlite):
+            self.pkgSack = sqlitesack.YumSqlitePackageSack(sqlitesack.YumAvailablePackageSqlite)
+        else:
+            self.pkgSack = YumPackageSack(YumAvailablePackage)
+            
     
     def __str__(self):
         return self.repos.keys()
     
 
-    def add(self, repoid):
-        if self.repos.has_key(repoid):
-            raise Errors.RepoError, 'Repository %s already added, not adding again' % (repoid)
-        thisrepo = Repository(repoid)
-        self.repos[repoid] = thisrepo
+    def add(self, repoobj):
+        if self.repos.has_key(repoobj.id):
+            raise Errors.RepoError, 'Repository %s already added, not adding again' % (repoobj.id)
+        self.repos[repoobj.id] = repoobj
         
-        return thisrepo
 
     def delete(self, repoid):
         if self.repos.has_key(repoid):
@@ -127,42 +134,15 @@ class RepoStorage:
             raise Errors.RepoError, \
                 'Error getting repository data for %s, repository not found' % (repoid)
 
-    def findRepos(self,pattern):
-        """find all repositories matching fnmatch `pattern`"""
-
-        result = []
-        match = re.compile(fnmatch.translate(pattern)).match
-
-        for name,repo in self.repos.items():
-            if match(name):
-                result.append(repo)
-        return result
-
-
     def disableRepo(self, repoid):
-        """disable a repository from use
-
-        fnmatch wildcards may be used to disable a group of repositories.
-        """
-        if _is_fnmatch_pattern(repoid):
-            for repo in self.findRepos(repoid):
-                repo.disable()
-        else:
-            thisrepo = self.getRepo(repoid)
-            thisrepo.disable()
-
+        """disable a repository from use"""
+        thisrepo = self.getRepo(repoid)
+        thisrepo.disable()
+            
     def enableRepo(self, repoid):
-        """enable a repository for use
-
-        fnmatch wildcards may be used to enable a group of repositories.
-        """
-        if _is_fnmatch_pattern(repoid):
-            repos = self.findRepos(repoid)
-            for repo in self.findRepos(repoid):
-                repo.enable()
-        else:
-            thisrepo = self.getRepo(repoid)
-            thisrepo.enable()
+        """disable a repository from use"""
+        thisrepo = self.getRepo(repoid)
+        thisrepo.enable()
             
     def listEnabled(self):
         """return list of enabled repo objects"""
@@ -228,10 +208,20 @@ class RepoStorage:
             data = ['metadata', 'filelists', 'otherdata']
         else:
             data = [ with ]
-        
+         
         for repo in myrepos:
             if not hasattr(repo, 'cacheHandler'):
-                repo.cacheHandler = mdcache.RepodataParser(storedir=repo.cachedir, callback=callback)
+                if (self.sqlite):
+                    repo.cacheHandler = self.sqlitecache.RepodataParserSqlite(
+                            storedir=repo.cachedir, 
+                            repoid=repo.id,
+                            callback=callback,
+                            )
+                else:
+                    repo.cacheHandler = mdcache.RepodataParser(
+                            storedir=repo.cachedir, 
+                            callback=callback
+                            )
             for item in data:
                 if self.pkgSack.added.has_key(repo.id):
                     if item in self.pkgSack.added[repo.id]:
@@ -422,7 +412,7 @@ class Repository:
             self.proxy_dict['http'] = proxy_string
             self.proxy_dict['https'] = proxy_string
             self.proxy_dict['ftp'] = proxy_string
-        
+
     def setupGrab(self):
         """sets up the grabber functions with the already stocked in urls for
            the mirror groups"""
@@ -524,7 +514,6 @@ class Repository:
         prxy = None
         if self.proxy_dict:
             prxy = self.proxy_dict
-
         if url is not None:
             ug = URLGrabber(keepalive = self.keepalive, 
                             bandwidth = self.bandwidth,
