@@ -12,7 +12,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-# Copyright 2004 Duke University 
+# Copyright 2005 Duke University 
 
 
 import os
@@ -64,8 +64,12 @@ def main(args):
         sys.exit(200)
 
     # build up an idea of what we're supposed to do
+    if base.basecmd == 'shell':
+        do = base.doShell
+    else:
+        do = base.doCommands
     try:
-        result, resultmsgs = base.doCommands()
+        result, resultmsgs = do()
     except Errors, e:
         result = 1
         resultmsgs = [str(e)]
@@ -168,7 +172,10 @@ def main(args):
     
     
 if __name__ == "__main__":
-    
+    #import hotshot
+    #p = hotshot.Profile(os.path.expanduser("~/yum.prof"))
+    #p.run('main(sys.argv[1:])')
+    #p.close()    
     try:
         main(sys.argv[1:])
     except KeyboardInterrupt, e:
